/*
zxor.c Cu{
coded by isaku@pb4.so-net.ne.jp
*/

#include <math.h>
#include "zxor.h"

/*+--------------------------------+
@|}NœnftHg̗̈|
@+--------------------------------+*/
struct _state_t DefaultMt[MSEQ];
volatile int32_t InitialSeed=-1;

/*+----------------+
@|̂`|
@+----------------+*/
static uint32_t InitMtSub(uint32_t s,uint32_t i)
{ return 1812433253UL*(s^(s>>30))+i+1; }

/*+------------+
@|̈|
@+------------+*/
void InitMt_r(state_p mt,uint32_t s)
{
    int i;

    for (i=0;i<4;i++) mt->u.x[i]=s=InitMtSub(s,i);
    mt->index=4; mt->range=0; mt->normal_sw=0;
    mt->coin_bits=0; mt->byte_pos=0;
}

/*+------------------+
@|zgď|
@+------------------+*/
void InitMtEx_r(state_p mt,uint32_t*key,int len)
{
    int i,k; uint32_t*x=mt->u.x,s;

    InitMt_r(mt,1); s=x[3];
    for (i=0;i<len;i++) x[i&3]^=s=InitMtSub(s,key[i]+i);
    for (k=0;k<3;k++,i++) x[i&3]^=s=InitMtSub(s,k);
    if (x[0]==0&&x[1]==0&&x[2]==0&&x[3]==0) x[0]=1;
}

/*+--------------+
@|RXgN^|
@+--------------+*/
#ifdef __cplusplus
_state_t::_state_t()
{
    static volatile int32_t seed=-1;

    if (this==DefaultMt) InitMt_r(this,1);
    else InitMt_r(this,seed--);
}
#endif

/*+------------------------------------------------------+
@|ۂߌ덷̂ȂOȏrange̐@@@@@@@ |
@|@@@@@@@@@@@@@@@@@@@@@@@@@@@|
@|rangeȂNextInt_r()T{`POB@@@|
@|for (i=0;i<10000;i++)@@@@@@@@@@ @@@@@@|
@|{ x[i]=NextIntEx(100); y[i]=NextIntEx(200); } @@@@|
@|ƂA@@@@@@@@@@@@@@@@@@@@|
@|for (i=0;i<10000;i++) x[i]=NextIntEx(100);@@@@@@|
@|for (i=0;i<10000;i++) y[i]=NextIntEx(200);@@@@@@|
@|ƂقPO{ȏ㍂ɂȂB܂ng|
@|for (i=0;i<10000;i++)@@@@@@@@@@ @@@@@@|
@|{ x[i]=NextIntEx_m(1,100); y[i]=NextIntEx_m(2,200); } |
@|ƂĂPO{ȏ㍂ɂȂ@@@@@@@@@@@@@|
@| range ςȂA@@@@@@@@ |
@|NextInt_r() ɔׂQ{̎Ԃ@@@@@@@@@|
@+------------------------------------------------------+*/
int32_t NextIntEx_r(state_p mt,int32_t range)
{
    uint32_t y,base,remain; int shift;

    if (range<=0) return 0;
    if (range!=mt->range) {
        mt->range=base=range;
        for (shift=0;base<=(1UL<<30);shift++) base<<=1;
        mt->base=base; mt->shift=shift;
    }
    for (;;) {
        y=NextMt_r(mt)>>1;
        if (y<mt->base) return(int32_t)(y>>mt->shift);
        base=mt->base; shift=mt->shift; y-=base;
        remain=(1UL<<31)-base;
        for (;remain>=(uint32_t)range;remain-=base) {
            for (;base>remain;base>>=1) shift--;
            if (y<base) return(int32_t)(y>>shift); else y-=base;
        }
    }
}

int32_t NextIntEx(int32_t range)
{
    state_p mt=DefaultMt; uint32_t y,base,remain; int shift;

    if (range<=0) return 0;
    if (range!=mt->range) {
        mt->range=base=range;
        for (shift=0;base<=(1UL<<30);shift++) base<<=1;
        mt->base=base; mt->shift=shift;
    }
    for (;;) {
        y=NextMt()>>1;
        if (y<mt->base) return(int32_t)(y>>mt->shift);
        base=mt->base; shift=mt->shift; y-=base;
        remain=(1UL<<31)-base;
        for (;remain>=(uint32_t)range;remain-=base) {
            for (;base>remain;base>>=1) shift--;
            if (y<base) return(int32_t)(y>>shift); else y-=base;
        }
    }
}

/*+----------------------------+
@|Rx˂̃JCQ敪z p.27 |
@+----------------------------+*/
double NextChisq_r(state_p mt,double n)
{
    return 2*NextGamma_r(mt,0.5*n);
}

/*+------------------------------+
@|p[^̃K}z p.31 |
@+------------------------------+*/
double NextGamma_r(state_p mt,double a)
{
    double t,u,x,y;
    if (a>1) {
        t=sqrt(2*a-1);
        do {
            do {
                do {
                    x=1-NextUnif_r(mt);
                    y=2*NextUnif_r(mt)-1;
                } while (x*x+y*y>1);
                y/=x; x=t*y+a-1;
            } while (x<=0);
            u=(a-1)*log(x/(a-1))-t*y;
        } while (u<-50||NextUnif_r(mt)>(1+y*y)*exp(u));
    } else {
        t=2.718281828459045235/(a+2.718281828459045235);
        do {
            if (NextUnif_r(mt)<t) {
                x=pow(NextUnif_r(mt),1/a); y=exp(-x);
            } else {
                x=1-log(1-NextUnif_r(mt)); y=pow(x,a-1);
            }
        } while (NextUnif_r(mt)>=y);
    }
    return x;
}

/*+----------------------+
@|mo̊􉽕z p.34 |
@+----------------------+*/
int NextGeometric_r(state_p mt,double p)
{ return(int)ceil(log(1.0-NextUnif_r(mt))/log(1-p)); }

/*+--------------+
@|Opz p.89 |
@+--------------+*/
double NextTriangle_r(state_p mt)
{ double a=NextUnif_r(mt),b=NextUnif_r(mt); return a-b; }

/*+----------------------+
@|ςP̎wz p.106|
@+----------------------+*/
double NextExp_r(state_p mt)
{ return-log(1-NextUnif_r(mt)); }

/*+----------------------------------+
@|WKz(ő6.660437) p.133|
@+----------------------------------+*/
double NextNormal_r(state_p mt)
{
    if (mt->normal_sw==0) {
        double t=sqrt(-2*log(1.0-NextUnif_r(mt)));
        double u=3.141592653589793*2*NextUnif_r(mt);
        mt->normal_save=t*sin(u); mt->normal_sw=1; return t*cos(u);
    }else{ mt->normal_sw=0; return mt->normal_save; }
}

/*+----------------------------------+
@|m̃_PʃxNg p.185|
@+----------------------------------+*/
void NextUnitVect_r(state_p mt,double*v,int n)
{
    int i; double r=0;
    for (i=0;i<n;i++) { v[i]=NextNormal_r(mt); r+=v[i]*v[i]; }
    if (r==0.0) r=1.0;
    r=sqrt(r);
    for (i=0;i<n;i++) v[i]/=r;
}

/*+--------------------------------+
@|p[^m,ôQz p.203 |
@+--------------------------------+*/
int NextBinomial_r(state_p mt,int n,double p)
{
    int i,r=0;
    for (i=0;i<n;i++) if (NextUnif_r(mt)<p) r++;
    return r;
}

/*+--------------------------------+
@|֌Wq̂QϗʐKz p.211|
@+--------------------------------+*/
void NextBinormal_r(state_p mt,double r,double*x,double*y)
{
    double r1,r2,s;
    do {
        r1=2*NextUnif_r(mt)-1;
        r2=2*NextUnif_r(mt)-1;
        s=r1*r1+r2*r2;
    } while (s>1||s==0);
    s= -log(s)/s; r1=sqrt((1+r)*s)*r1;
    r2=sqrt((1-r)*s)*r2; *x=r1+r2; *y=r1-r2;
}

/*+----------------------------------+
@|p[^`,ãx[^z p.257 |
@+----------------------------------+*/
double NextBeta_r(state_p mt,double a,double b)
{
    double temp=NextGamma_r(mt,a);
    return temp/(temp+NextGamma_r(mt,b));
}

/*+----------------------------+
@|p[^m̗ݏ敪z p.305|
@+----------------------------+*/
double NextPower_r(state_p mt,double n)
{ return pow(NextUnif_r(mt),1.0/(n+1)); }

/*+------------------------+
@|WXeBbNz p.313|
@+------------------------+*/
double NextLogistic_r(state_p mt)
{
    double r;
    do r=NextUnif_r(mt); while (r==0);
    return log(r/(1-r));
}

/*+------------------+
@|R[V[z p.331|
@+------------------+*/
double NextCauchy_r(state_p mt)
{
    double x,y;
    do { x=1-NextUnif_r(mt); y=2*NextUnif_r(mt)-1; }
    while (x*x+y*y>1);
    return y/x;
}

/*+--------------------------+
@|Rx`,âez p.344 |
@+--------------------------+*/
double NextFDist_r(state_p mt,double n1,double n2)
{
    double nc1=NextChisq_r(mt,n1),nc2=NextChisq_r(mt,n2);
    return (nc1*n2)/(nc2*n1);
}

/*+--------------------------+
@|σɂ̃|A\z p.412|
@+--------------------------+*/
int NextPoisson_r(state_p mt,double lambda)
{
    int k; lambda=exp(lambda)*NextUnif_r(mt);
    for (k=0;lambda>1;k++) lambda*=NextUnif_r(mt);
    return k;
}

/*+----------------------+
@|Rxm̂z p.428|
@+----------------------+*/
double NextTDist_r(state_p mt,double n)
{
    double a,b,c;
    if (n<=2) {
        do a=NextChisq_r(mt,n); while (a==0);
        return NextNormal_r(mt)/sqrt(a/n);
    }
    do {
        a=NextNormal_r(mt); b=a*a/(n-2);
        c=log(1-NextUnif_r(mt))/(1-0.5*n);
    } while (exp(-b-c)>1-b);
    return a/sqrt((1-2.0/n)*(1-b));
}

/*+--------------------------------+
@|p[^̃Cuz p.431|
@+--------------------------------+*/
double NextWeibull_r(state_p mt,double alpha)
{ return pow(-log(1-NextUnif_r(mt)),1/alpha); }
